use criterion::{black_box, criterion_group, criterion_main, Criterion};

use microrm::{make_index, Entity};
use rand::prelude::*;
use serde::{Deserialize, Serialize};

#[global_allocator]
static GLOBAL: &stats_alloc::StatsAlloc<std::alloc::System> = &stats_alloc::INSTRUMENTED_SYSTEM;

#[derive(Entity, serde::Serialize, serde::Deserialize)]
pub struct RowIDIndexed {
    val: usize,
}

fn rowid_indexed(c: &mut Criterion) {
    let schema = microrm::model::SchemaModel::new().entity::<RowIDIndexed>();
    let db = microrm::DB::new_in_memory(schema).unwrap();
    let qi = db.query_interface();

    for i in 0..10000 {
        qi.add(&RowIDIndexed { val: i * i });
    }

    let mut reg = stats_alloc::Region::new(&GLOBAL);

    qi.get_one_by_id(RowIDIndexedID(0));

    println!(
        "allocations for single get_one_by_id: {}",
        reg.change_and_reset().allocations
    );

    c.bench_function("select 0", |b| {
        b.iter(|| qi.get_one_by_id(RowIDIndexedID(0)))
    });
    c.bench_function("select 0 no statement cache", |b| {
        b.iter(|| db.query_interface().get_one_by_id(RowIDIndexedID(0)))
    });

    fn random_id() -> RowIDIndexedID {
        let res = RowIDIndexedID((rand::random::<usize>() % 10000) as i64);
        res
    }

    c.bench_function("select random", |b| {
        b.iter(|| qi.get_one_by_id(random_id()))
    });
}

criterion_group!(benches, rowid_indexed);
criterion_main!(benches);
